==> scanbot_sdk-2.2.0-beta1.tar.gz: android/src/main/kotlin/io/scanbot/sdk/flutter/PageProcessorPlugin.kt <==
/*
    Scanbot SDK Flutter Plugin

    Copyright (c) 2019 doo GmbH. All rights reserved.
 */
package io.scanbot.sdk.flutter

import android.content.Context
import android.net.Uri
import com.google.gson.*
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel.Result
import io.scanbot.sdk.ScanbotSDK
import io.scanbot.sdk.core.contourdetector.DetectionResult
import io.scanbot.sdk.entity.Language
import io.scanbot.sdk.flutter.utils.StorageUtils
import io.scanbot.sdk.persistence.Page
import io.scanbot.sdk.persistence.PageFileStorage
import io.scanbot.sdk.process.ImageFilterType
import io.scanbot.sdk.process.PDFPageSize
import io.scanbot.sdk.tiff.model.TIFFImageWriterCompressionOptions
import io.scanbot.sdk.tiff.model.TIFFImageWriterParameters
import io.scanbot.sdk.tiff.model.TIFFImageWriterUserDefinedField
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.async
import kotlinx.coroutines.awaitAll
import kotlinx.coroutines.withContext
import org.json.JSONObject
import java.io.File
import java.util.*

class PageProcessorPlugin(
        val context: Context,
        coroutineResultHandler: CoroutineResultHandler,
        val scanbotSDK: ScanbotSDK
) : ISdkSubplugin,
        ICoroutineActivityResultHandler by coroutineResultHandler {

    private val gson = GsonBuilder()
            .setFieldNamingPolicy(FieldNamingPolicy.IDENTITY)
            .create()
    private val pageProcessor = scanbotSDK.pageProcessor()
    private val ocrRecognizer = scanbotSDK.ocrRecognizer()
    private val pageFileStorage = scanbotSDK.pageFileStorage()
    override suspend fun onMethodCall(call: MethodCall, result: Result): Boolean {
        when (call.method) {
            "applyImageFilter" -> applyImageFilter(call, result)
            "rotatePageClockwise" -> rotatePageClockwise(call, result)
            "performOcr" -> performOcr(context, call, result)
            "getOcrConfig" -> getOcrConfigs(call, result)
            "detectDocument" -> detectDocument(call, result)
            "createPage" -> createPage(context, call, result)
            "createTiff" -> createTiff(context, call, result)
            "createPdf" -> createPdf(context, call, result)
            "refreshImageUris" -> refreshImageUris(context, call, result)
            else -> return false
        }
        return true
    }

    private suspend fun applyImageFilter(
            call: MethodCall,
            methodResult: Result
    ) {
        withContext(Dispatchers.Default) {
            try {
                val pageJson = call.argument<PageJson>("page") ?: PageJson()
                val imageFilterType = call.argument<String>("filter") ?: ""
                val pageFromJson = pageJson.toPage(gson)

                val page = pageProcessor.applyFilter(
                        pageFromJson,
                        ImageFilterType.valueOf(imageFilterType)
                )
                val json = gson.toJsonTree(page) as JsonObject
                insertPageUris(scanbotSDK, this, page.pageId, json)
                withContext(Dispatchers.Main) { methodResult.success(json.toString()) }
            } catch (e: Throwable) {
                withContext(Dispatchers.Main) {
                    methodResult.error(
                            e::class.java.simpleName,
                            e.message,
                            null
                    )
                }
            }
        }
    }

    private suspend fun rotatePageClockwise(
            call: MethodCall,
            methodResult: Result
    ) {
        withContext(Dispatchers.Default) {
            try {
                val pageJson =
                        call.argument<PageJson>("page") ?: PageJson()
                val times =
                        call.argument<Int>("rotationsCount") ?: 1

                val pageFromJson = pageJson.toPage(gson)

                val page = pageProcessor.rotate(
                        pageFromJson,
                        times
                )
                val json = gson.toJsonTree(page) as JsonObject
                insertPageUris(scanbotSDK, this, page.pageId, json)

                withContext(Dispatchers.Main) {
                    methodResult.success(json.toString())
                }
            } catch (e: Throwable) {
                withContext(Dispatchers.Main) {
                    methodResult.error(
                            e::class.java.simpleName,
                            e.message,
                            null
                    )
                }
            }
        }
    }

    private suspend fun detectDocument(
            call: MethodCall,
            methodResult: Result
    ) {
        withContext(Dispatchers.Default) {
            try {
                val pageJson =
                        call.argument<PageJson>("page") ?: PageJson()
                var page = pageJson.toPage(gson)

                // run auto document detection on it:
                page = pageProcessor.detectDocument(page)

                val json = gson.toJsonTree(page) as JsonObject
                insertPageUris(scanbotSDK, this, page.pageId, json)
                withContext(Dispatchers.Main) {
                    methodResult.success(json.toString())
                }
            } catch (e: Throwable) {
                withContext(Dispatchers.Main) {
                    methodResult.error(
                            e::class.java.simpleName,
                            e.message,
                            null
                    )
                }
            }
        }
    }

    private suspend fun createPage(
            activity: Context,
            call: MethodCall,
            methodResult: Result
    ) {
        withContext(Dispatchers.Default) {
            try {
                val originalImageFileUri = call.argument<String>("originalImageFileUri") ?: ""
                val shouldDetectDocument = call.argument<Boolean>("shouldDetectDocument") ?: false

                val originalBitmap = prepareImageForImport(Uri.parse(originalImageFileUri), activity, fixOrientation = true)
                val pageId = pageFileStorage.add(originalBitmap)
                var page = Page(pageId, emptyList(), DetectionResult.OK, ImageFilterType.NONE)

                if (shouldDetectDocument) {
                    // run auto document detection on it:
                    page = pageProcessor.detectDocument(page)
                }

                val json = gson.toJsonTree(page) as JsonObject
                insertPageUris(scanbotSDK, this, page.pageId, json)
                withContext(Dispatchers.Main) {
                    methodResult.success(json.toString())
                }
            } catch (e: Throwable) {
                withContext(Dispatchers.Main) {
                    methodResult.error(
                            e::class.java.simpleName,
                            e.message,
                            null
                    )
                }
            }
        }
    }

    private suspend fun getOcrConfigs(
            call: MethodCall,
            methodResult: Result
    ) {
        withContext(Dispatchers.Default) {
            try {
                val configJson = JsonObject()
                val languagesJson = JsonArray()
                ocrRecognizer.getInstalledLanguages().map {
                    it.languageTags.firstOrNull() ?: it.ocrBlobLanguageTag
                }.forEach {
                    languagesJson.add(it)
                }
                configJson.addProperty("languageDataPath", ocrRecognizer.getLanguageDataPath().path)
                configJson.add("installedLanguages", languagesJson)
                withContext(Dispatchers.Main) {
                    methodResult.success(configJson.toString())
                }
            } catch (e: Throwable) {
                withContext(Dispatchers.Main) {
                    methodResult.error(
                            e::class.java.simpleName,
                            e.message,
                            null
                    )
                }
            }
        }
    }

    private suspend fun performOcr(
            context: Context,
            call: MethodCall,
            methodResult: Result
    ) {
        withContext(Dispatchers.Default) {
            try {
                val pagesJson = call.argument<PagesJson>("pages")!!
                val options = call.argument<OcrOptionsJson>("options")

                val languages = (options?.get("languages") as LanguagesJson).mapNotNull {
                    Language.languageByLanguageTag(it)
                }.toSet()

                val shouldGeneratePdf = options?.get("shouldGeneratePdf") as? Boolean
                val pages =
                        pagesJson.map { gson.fromJson(JSONObject(it).toString(), Page::class.java) }
                                .toList()

                val ocrResult = if (shouldGeneratePdf == true) {
                    ocrRecognizer.recognizeTextWithPdfFromPages(pages, PDFPageSize.AUTO, languages)
                } else {
                    ocrRecognizer.recognizeTextFromPages(pages, languages)
                }
                val tmpFile = ocrResult?.sandwichedPdfDocumentFile
                val newFile = tmpFile?.let {
                    val newFile = movePdfFileToStorage(it, context)
                    newFile
                }

                val ocrResultJson = ocrResultToJson(ocrResult, newFile)
                withContext(Dispatchers.Main) {
                    methodResult.success(ocrResultJson.toString())
                }
            } catch (e: Throwable) {
                withContext(Dispatchers.Main) {
                    methodResult.error(
                            e::class.java.simpleName,
                            e.message,
                            null
                    )
                }
            }
        }
    }

    private suspend fun createTiff(
            activity: Context,
            call: MethodCall,
            methodResult: Result
    ) {
        withContext(Dispatchers.Default) {
            try {
                val pagesJson = call.argument<PagesJson>("pages")!!
                val options = call.argument<TiffOptionsJson>("options")
                val binarized = options?.get("binarized") as Boolean
                val compressionString = (options?.get("compression") as? String)
                        .takeUnless { it.isNullOrBlank() }
                        ?: "NONE"
                val imageFiles =
                        pagesJson.map { gson.fromJson(JSONObject(it).toString(), Page::class.java) }
                                .map {
                                    async {
                                        pageFileStorage.getImageURI(
                                                it.pageId,
                                                PageFileStorage.PageFileType.DOCUMENT
                                        )
                                    }
                                }.awaitAll().map {
                                    File(it.path)
                                }


                val dpi = (options?.get("dpi") as? Int) ?: 200

                val compression = TIFFImageWriterCompressionOptions.valueOf("COMPRESSION_$compressionString")


                // no custom fields supported yet.
                val customFields = ArrayList<TIFFImageWriterUserDefinedField>()

                val params = TIFFImageWriterParameters(if (binarized) ImageFilterType.PURE_BINARIZED else ImageFilterType.NONE, dpi, compression, customFields)

                val tiffTargetFile = StorageUtils.generateRandomPluginStorageFile("tiff", activity)
                val tiffWriter = scanbotSDK.tiffWriter()
                tiffWriter.writeTIFFFromFiles(imageFiles, tiffTargetFile, params)

                withContext(Dispatchers.Main) {
                    methodResult.success(Uri.fromFile(tiffTargetFile).toString())
                }
            } catch (e: Throwable) {
                withContext(Dispatchers.Main) {
                    methodResult.error(
                            e::class.java.simpleName,
                            e.message,
                            null
                    )
                }
            }
        }
    }

    private suspend fun createPdf(
            context: Context,
            call: MethodCall,
            methodResult: Result
    ) {
        withContext(Dispatchers.Default) {
            try {
                val pagesJson = call.argument<PagesJson>("pages")!!
                val options = call.argument<PdfOptionsJson>("options")
                val value = (options?.get("renderPageSize") as? String) ?: PDFPageSize.AUTO.name
                val pdfPageSize = PDFPageSize.valueOf(value)
                val pages = pagesJson.map { gson.fromJson(JSONObject(it).toString(), Page::class.java) }
                val pdfRenderer = scanbotSDK.pdfRenderer()
                val tmpfile = pdfRenderer.renderDocumentFromPages(pages, pdfPageSize)
                val file = movePdfFileToStorage(tmpfile, context)
                withContext(Dispatchers.Main) {
                    methodResult.success(Uri.fromFile(file).toString())
                }
            } catch (e: Throwable) {
                withContext(Dispatchers.Main) {
                    methodResult.error(
                            e::class.java.simpleName,
                            e.message,
                            null
                    )
                }
            }
        }
    }

    private suspend fun movePdfFileToStorage(tempPdfFile: File?, context: Context): File {
        try {
            val pdfOutputUri = Uri.fromFile(StorageUtils.generateRandomPluginStorageFile("pdf", context))
            val newFile = File(pdfOutputUri.path)
            tempPdfFile?.let { StorageUtils.copyFile(it, newFile) }
            return newFile
        } catch (e: Throwable) {
            throw e
        } finally {
            tempPdfFile?.let { StorageUtils.forceDelete(it) }
        }
    }

    private suspend fun refreshImageUris(context: Context, call: MethodCall, methodResult: Result) {
        // This method is actually not required on Android. We just provide a dummy implementation
        // to stay consistent with the Flutter API and iOS. See the comments of the iOS implementation.
        withContext(Dispatchers.Default) {
            try {
                val pagesJson = call.argument<PagesJson>("pages")!!
                val array = JsonArray()
                pagesJson.map { array.add(gson.toJsonTree(it)) }
                val resultJson = JsonObject()
                resultJson.add("pages", array)
                withContext(Dispatchers.Main) { methodResult.success(resultJson.toString()) }
            } catch (e: Throwable) {
                withContext(Dispatchers.Main) {
                    methodResult.error(
                            e::class.java.simpleName,
                            e.message,
                            null
                    )
                }
            }
        }
    }
}

==> scanbot_sdk-2.3.0-beta1.tar.gz: android/src/main/kotlin/io/scanbot/sdk/flutter/PageProcessorPlugin.kt <==
/*
    Scanbot SDK Flutter Plugin

    Copyright (c) 2019 doo GmbH. All rights reserved.
 */
package io.scanbot.sdk.flutter

import android.content.Context
import android.net.Uri
import com.google.gson.*
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel.Result
import io.scanbot.sdk.ScanbotSDK
import io.scanbot.sdk.core.contourdetector.DetectionResult
import io.scanbot.sdk.entity.Language
import io.scanbot.sdk.flutter.utils.StorageUtils
import io.scanbot.sdk.persistence.Page
import io.scanbot.sdk.persistence.PageFileStorage
import io.scanbot.sdk.process.ImageFilterType
import io.scanbot.sdk.process.PDFPageSize
import io.scanbot.sdk.tiff.model.TIFFImageWriterCompressionOptions
import io.scanbot.sdk.tiff.model.TIFFImageWriterParameters
import io.scanbot.sdk.tiff.model.TIFFImageWriterUserDefinedField
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.async
import kotlinx.coroutines.awaitAll
import kotlinx.coroutines.withContext
import org.json.JSONObject
import java.io.File
import java.util.*

class PageProcessorPlugin(
        val context: Context,
        coroutineResultHandler: CoroutineResultHandler,
        val scanbotSDK: ScanbotSDK,
        val isEncryptionEnabled: Boolean
) : ISdkSubplugin,
        ICoroutineActivityResultHandler by coroutineResultHandler {

    private val gson = GsonBuilder()
            .setFieldNamingPolicy(FieldNamingPolicy.IDENTITY)
            .create()
    private val pageProcessor = scanbotSDK.pageProcessor()
    private val ocrRecognizer = scanbotSDK.ocrRecognizer()
    private val pageFileStorage = scanbotSDK.pageFileStorage()
    override suspend fun onMethodCall(call: MethodCall, result: Result): Boolean {
        when (call.method) {
            "applyImageFilter" -> applyImageFilter(call, result)
            "rotatePageClockwise" -> rotatePageClockwise(call, result)
            "performOcr" -> performOcr(context, call, result)
            "getOcrConfig" -> getOcrConfigs(call, result)
            "detectDocument" -> detectDocument(call, result)
            "createPage" -> createPage(context, call, result)
            "createTiff" -> createTiff(context, call, result)
            "createPdf" -> createPdf(context, call, result)
            "refreshImageUris" -> refreshImageUris(context, call, result)
            else -> return false
        }
        return true
    }

    private suspend fun applyImageFilter(
            call: MethodCall,
            methodResult: Result
    ) {
        withContext(Dispatchers.Default) {
            try {
                val pageJson = call.argument<PageJson>("page") ?: PageJson()
                val imageFilterType = call.argument<String>("filter") ?: ""
                val pageFromJson = pageJson.toPage(gson)

                val page = pageProcessor.applyFilter(
                        pageFromJson,
                        ImageFilterType.valueOf(imageFilterType)
                )
                val json = gson.toJsonTree(page) as JsonObject
                insertPageUris(scanbotSDK, this, page.pageId, json)
                withContext(Dispatchers.Main) { methodResult.success(json.toString()) }
            } catch (e: Throwable) {
                withContext(Dispatchers.Main) {
                    methodResult.error(
                            e::class.java.simpleName,
                            e.message,
                            null
                    )
                }
            }
        }
    }

    private suspend fun rotatePageClockwise(
            call: MethodCall,
            methodResult: Result
    ) {
        withContext(Dispatchers.Default) {
            try {
                val pageJson =
                        call.argument<PageJson>("page") ?: PageJson()
                val times =
                        call.argument<Int>("rotationsCount") ?: 1

                val pageFromJson = pageJson.toPage(gson)

                val page = pageProcessor.rotate(
                        pageFromJson,
                        times
                )
                val json = gson.toJsonTree(page) as JsonObject
                insertPageUris(scanbotSDK, this, page.pageId, json)

                withContext(Dispatchers.Main) {
                    methodResult.success(json.toString())
                }
            } catch (e: Throwable) {
                withContext(Dispatchers.Main) {
                    methodResult.error(
                            e::class.java.simpleName,
                            e.message,
                            null
                    )
                }
            }
        }
    }

    private suspend fun detectDocument(
            call: MethodCall,
            methodResult: Result
    ) {
        withContext(Dispatchers.Default) {
            try {
                val pageJson =
                        call.argument<PageJson>("page") ?: PageJson()
                var page = pageJson.toPage(gson)

                // run auto document detection on it:
                page = pageProcessor.detectDocument(page)

                val json = gson.toJsonTree(page) as JsonObject
                insertPageUris(scanbotSDK, this, page.pageId, json)
                withContext(Dispatchers.Main) {
                    methodResult.success(json.toString())
                }
            } catch (e: Throwable) {
                withContext(Dispatchers.Main) {
                    methodResult.error(
                            e::class.java.simpleName,
                            e.message,
                            null
                    )
                }
            }
        }
    }

    private suspend fun createPage(
            activity: Context,
            call: MethodCall,
            methodResult: Result
    ) {
        withContext(Dispatchers.Default) {
            try {
                val originalImageFileUri = call.argument<String>("originalImageFileUri") ?: ""
                val shouldDetectDocument = call.argument<Boolean>("shouldDetectDocument") ?: false

                val originalBitmap = prepareImageForImport(Uri.parse(originalImageFileUri), activity, fixOrientation = true)
                val pageId = pageFileStorage.add(originalBitmap)
                var page = Page(pageId, emptyList(), DetectionResult.OK, ImageFilterType.NONE)

                if (shouldDetectDocument) {
                    // run auto document detection on it:
                    page = pageProcessor.detectDocument(page)
                }

                val json = gson.toJsonTree(page) as JsonObject
                insertPageUris(scanbotSDK, this, page.pageId, json)
                withContext(Dispatchers.Main) {
                    methodResult.success(json.toString())
                }
            } catch (e: Throwable) {
                withContext(Dispatchers.Main) {
                    methodResult.error(
                            e::class.java.simpleName,
                            e.message,
                            null
                    )
                }
            }
        }
    }

    private suspend fun getOcrConfigs(
            call: MethodCall,
            methodResult: Result
    ) {
        withContext(Dispatchers.Default) {
            try {
                val configJson = JsonObject()
                val languagesJson = JsonArray()
                ocrRecognizer.getInstalledLanguages().map {
                    it.languageTags.firstOrNull() ?: it.ocrBlobLanguageTag
                }.forEach {
                    languagesJson.add(it)
                }
                configJson.addProperty("languageDataPath", ocrRecognizer.getLanguageDataPath().path)
                configJson.add("installedLanguages", languagesJson)
                withContext(Dispatchers.Main) {
                    methodResult.success(configJson.toString())
                }
            } catch (e: Throwable) {
                withContext(Dispatchers.Main) {
                    methodResult.error(
                            e::class.java.simpleName,
                            e.message,
                            null
                    )
                }
            }
        }
    }

    private suspend fun performOcr(
            context: Context,
            call: MethodCall,
            methodResult: Result
    ) {
        withContext(Dispatchers.Default) {
            try {
                val pagesJson = call.argument<PagesJson>("pages")!!
                val options = call.argument<OcrOptionsJson>("options")

                val languages = (options?.get("languages") as LanguagesJson).mapNotNull {
                    Language.languageByLanguageTag(it)
                }.toSet()

                val shouldGeneratePdf = options?.get("shouldGeneratePdf") as? Boolean
                val pages =
                        pagesJson.map { gson.fromJson(JSONObject(it as Map<*, *>).toString(), Page::class.java) }
                                .toList()

                val ocrResult = if (shouldGeneratePdf == true) {
                    ocrRecognizer.recognizeTextWithPdfFromPages(pages, PDFPageSize.AUTO, languages)
                } else {
                    ocrRecognizer.recognizeTextFromPages(pages, languages)
                }
                val tmpFile = ocrResult?.sandwichedPdfDocumentFile
                val newFile = tmpFile?.let {
                    val newFile = movePdfFileToStorage(it, context)
                    newFile
                }

                val ocrResultJson = ocrResultToJson(ocrResult, newFile)
                withContext(Dispatchers.Main) {
                    methodResult.success(ocrResultJson.toString())
                }
            } catch (e: Throwable) {
                withContext(Dispatchers.Main) {
                    methodResult.error(
                            e::class.java.simpleName,
                            e.message,
                            null
                    )
                }
            }
        }
    }

    private suspend fun createTiff(
            activity: Context,
            call: MethodCall,
            methodResult: Result
    ) {
        withContext(Dispatchers.Default) {
            try {
                val pagesJson = call.argument<PagesJson>("pages")!!
                val options = call.argument<TiffOptionsJson>("options")
                val binarized = options?.get("binarized") as Boolean
                val compressionString = (options?.get("compression") as? String)
                        .takeUnless { it.isNullOrBlank() }
                        ?: "NONE"

                val imageFiles =
                        pagesJson.map { gson.fromJson(JSONObject(it as Map<*, *>).toString(), Page::class.java) }
                                .map {
                                    async {
                                        pageFileStorage.getImageURI(
                                                it.pageId,
                                                PageFileStorage.PageFileType.DOCUMENT
                                        )
                                    }
                                }.awaitAll().map {
                                    File(it.path)
                                }


                val dpi = (options?.get("dpi") as? Int) ?: 200

                val compression = TIFFImageWriterCompressionOptions.valueOf("COMPRESSION_$compressionString")


                // no custom fields supported yet.
                val customFields = ArrayList<TIFFImageWriterUserDefinedField>()

                val params = TIFFImageWriterParameters(if (binarized) ImageFilterType.PURE_BINARIZED else ImageFilterType.NONE, dpi, compression, customFields)

                val tiffTargetFile = StorageUtils.generateRandomPluginStorageFile("tiff", activity)
                val tiffWriter = scanbotSDK.tiffWriter()
                tiffWriter.writeTIFFFromFiles(imageFiles, isEncryptionEnabled, tiffTargetFile, params)

                withContext(Dispatchers.Main) {
                    methodResult.success(Uri.fromFile(tiffTargetFile).toString())
                }
            } catch (e: Throwable) {
                withContext(Dispatchers.Main) {
                    methodResult.error(
                            e::class.java.simpleName,
                            e.message,
                            null
                    )
                }
            }
        }
    }

    private suspend fun createPdf(
            context: Context,
            call: MethodCall,
            methodResult: Result
    ) {
        withContext(Dispatchers.Default) {
            try {
                val pagesJson = call.argument<PagesJson>("pages")!!
                val options = call.argument<PdfOptionsJson>("options")
                val value = (options?.get("renderPageSize") as? String) ?: PDFPageSize.AUTO.name
                val pdfPageSize = PDFPageSize.valueOf(value)
                val pages = pagesJson.map { gson.fromJson(JSONObject(it as Map<*, *>).toString(), Page::class.java) }
                val pdfRenderer = scanbotSDK.pdfRenderer()
                val tmpfile = pdfRenderer.renderDocumentFromPages(pages, pdfPageSize)
                val file = movePdfFileToStorage(tmpfile, context)
                withContext(Dispatchers.Main) {
                    methodResult.success(Uri.fromFile(file).toString())
                }
            } catch (e: Throwable) {
                withContext(Dispatchers.Main) {
                    methodResult.error(
                            e::class.java.simpleName,
                            e.message,
                            null
                    )
                }
            }
        }
    }

    private suspend fun movePdfFileToStorage(tempPdfFile: File?, context: Context): File {
        try {
            val pdfOutputUri = Uri.fromFile(StorageUtils.generateRandomPluginStorageFile("pdf", context))
            val newFile = File(pdfOutputUri.path)
            tempPdfFile?.let { StorageUtils.copyFile(it, newFile) }
            return newFile
        } catch (e: Throwable) {
            throw e
        } finally {
            tempPdfFile?.let { StorageUtils.forceDelete(it) }
        }
    }

    private suspend fun refreshImageUris(context: Context, call: MethodCall, methodResult: Result) {
        // This method is actually not required on Android. We just provide a dummy implementation
        // to stay consistent with the Flutter API and iOS. See the comments of the iOS implementation.
        withContext(Dispatchers.Default) {
            try {
                val pagesJson = call.argument<PagesJson>("pages")!!
                val array = JsonArray()
                pagesJson.map { array.add(gson.toJsonTree(it)) }
                val resultJson = JsonObject()
                resultJson.add("pages", array)
                withContext(Dispatchers.Main) { methodResult.success(resultJson.toString()) }
            } catch (e: Throwable) {
                withContext(Dispatchers.Main) {
                    methodResult.error(
                            e::class.java.simpleName,
                            e.message,
                            null
                    )
                }
            }
        }
    }
}
